"""
Usage /opt/imunify360/venv/bin/python3 rules_checker.py <action>
    choose an action from ACTIONS map
    e.g.
    /opt/imunify360/venv/bin/python3 rules_checker.py recreate

    Actions:
        - `recreate` - recreates rules if needed and checks ipsets consistent
        - `clear` - waits RulesChecker stop and destroys all rules and ipsets

    Actions based on lazy_init plugin
        placed in im360.plugins.protector.lazy_init
"""

import asyncio
import sys

from defence360agent.model import tls_check
from im360.files import WHITELISTS, Index
from defence360agent.contracts.config import Model
from defence360agent.model import instance
from im360.internals.core import ip_versions

from defence360agent.internals import logger
from im360.plugins.protector.lazy_init import RulesChecker


async def recreate_rules(rc: RulesChecker):
    print("Recreating rules")
    await rc.recreate_rules_if_needed()
    await rc.check_ipsets_consistent()
    print("Completed")


async def clear_everything(rc: RulesChecker):
    print("Clear rules and ipsets")
    rc.should_stop()
    await rc.wait()
    await rc.clear_everything()
    print("IP sets and rules have been removed successfully.")


def setup_environment():
    logger.reconfigure()
    ip_versions.init()
    instance.db.init(Model.PATH)
    instance.db.execute_sql(
        "ATTACH '{}' AS resident".format(Model.RESIDENT_PATH)
    )
    Index.add_type(WHITELISTS, "whitelist/v2", 0o770, 0o660, all_zip=True)


ACTIONS = {"recreate": recreate_rules, "clear": clear_everything}


def main(action):
    if action not in ACTIONS:
        print(f'Invalid action. Choose from {", ".join(ACTIONS.keys())}')
        sys.exit(1)

    loop = asyncio.get_event_loop()
    tls_check.reset()
    setup_environment()
    rc = RulesChecker(loop)
    action = ACTIONS[action]
    loop.run_until_complete(action(rc))


if __name__ == "__main__":
    if len(sys.argv) != 2:
        print(
            "Please provide one action as command line argument."
            f" {', '.join(ACTIONS.keys())}"
        )
        sys.exit(1)

    main(sys.argv[1])
